###################################################################################################
# _filtermanager.py
#
# $Id: _filtermanager.py,v 1.1 2003/10/05 13:03:14 dnordmann Exp $
# $Name:  $
# $Author: dnordmann $
# $Revision: 1.1 $
#
# Implementation of class FilterManager (see below).
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from __future__ import nested_scopes
from App.Common import package_home
from Globals import HTMLFile
import ZPublisher.HTTPRequest
import copy
import os
import tempfile
import time
import urllib
# Product Imports.
import _blobfields
import _fileutil
import _globals


"""
################################################################################################
#
#   X M L   I M / E X P O R T
#
################################################################################################
"""

# -------------------------------------------------------------------------------------------------
#  _filtermanager.initConf:
# -------------------------------------------------------------------------------------------------
def initConf(self, REQUEST):
  filename = 'filter.conf.win32.xml'
  xmlfile = open(_fileutil.getOSPath(package_home(globals())+'/import/'+filename),'rb')
  importXml(self, xmlfile, REQUEST)
  # Return filename.
  return filename


# -------------------------------------------------------------------------------------------------
#  importXml
# -------------------------------------------------------------------------------------------------

def _importXml(self, item):
  itemType = item.get('type')
  itemOb = item.get('value')
  if itemType == 'filter':
    newId = itemOb.get('id')
    newAcquired = 0
    newName = itemOb.get('name')
    newFormat = itemOb.get('format')
    newContentType = itemOb.get('content_type')
    newDescription = itemOb.get('description','')
    newRoles = itemOb.get('roles',[])
    newMetaTypes = itemOb.get('meta_types',[])
    delFilter(self, newId)
    setFilter(self, newId, newAcquired, newName, newFormat, newContentType, newDescription, newRoles, newMetaTypes)
    for process in itemOb.get('processes',[]):
      newProcessId = process.get('id')
      newProcessFile = process.get('file')
      setFilterProcess(self, newId, newProcessId, newProcessFile)
  elif itemType == 'process':
    newId = itemOb.get('id')
    newAcquired = 0
    newName = itemOb.get('name')
    newCommand = itemOb.get('command')
    delProcess(self, newId)
    setProcess(self, newId, newAcquired, newName, newCommand)
  else:
    _globals.writeException(self,"[_importXml]: Unknown type >%s<"%itemType)

def importXml(self, xml, REQUEST=None):
  v = self.parseXmlString(xml)
  if type(v) is type([]):
    for item in v:
      id = _importXml(self,item)
  else:
    id = _importXml(self,v)

# -------------------------------------------------------------------------------------------------
#  exportXml
# -------------------------------------------------------------------------------------------------
def exportXml(self, REQUEST, RESPONSE):
  value = []
  ids = REQUEST.get('ids',[])
  for id in self.getFilterIds():
    if id in ids or len(ids) == 0:
      ob = self.getFilter(id)
      value.append({'type':'filter','value':ob})
  for id in self.getProcessIds():
    if id in ids or len(ids) == 0:
      ob = self.getProcess(id)
      value.append({'type':'process','value':ob})
  # XML.
  if len(value)==1:
    value = value[0]
  content_type = 'text/xml'
  filename = 'importExportFilter.xml'
  export = '<?xml version="1.0" encoding="ISO-8859-1"?>'
  export = export + self.toXmlString(value,1)
  RESPONSE.setHeader('Content-Type',content_type)
  RESPONSE.setHeader('Content-Disposition','inline;filename=%s'%filename)
  return export

"""
################################################################################################
#
#   P R O C E S S E S
#
################################################################################################
"""

# ----------------------------------------------------------------------------------------------
#  _filtermanager.getRawProcesses:
#
#  Returns raw dictionary of processes.
# ----------------------------------------------------------------------------------------------
def getRawProcesses(self):
  # Return attribute.
  return self.getConfProperty('ZMS.filter.processes',{})

# ----------------------------------------------------------------------------------------------
#  _filtermanager.setProcess:
# 
#  Set/add process specified by given Id.
# ----------------------------------------------------------------------------------------------
def setProcess(self, newId, newAcquired=0, newName='', newCommand=''):
  # Set.
  obs = getRawProcesses(self)
  ob = {}
  ob['acquired'] = newAcquired
  ob['name'] = newName
  ob['command'] = newCommand
  obs[newId] = ob
  # Set attribute.
  self.setConfProperty('ZMS.filter.processes',obs.copy())
  # Return with new id.
  return newId

# ----------------------------------------------------------------------------------------------
#  _filtermanager.delProcess:
# 
#  Delete process specified by given Id.
# ----------------------------------------------------------------------------------------------
def delProcess(self, id):
  # Delete.
  cp = getRawProcesses(self)
  obs = {}
  for key in cp.keys():
    if key != id:
      obs[key] = cp[key]
  # Set attribute.
  self.setConfProperty('ZMS.filter.processes',obs.copy())
  # Return with empty id.
  return ''


"""
################################################################################################
#
#   F I L T E R S
#
################################################################################################
"""

# ----------------------------------------------------------------------------------------------
#  _filtermanager.getRawFilters:
#
#  Returns raw dictionary of filters.
# ----------------------------------------------------------------------------------------------
def getRawFilters(self):
  # Return attribute.
  return self.getConfProperty('ZMS.filter.filters',{})

# ----------------------------------------------------------------------------------------------
#  _filtermanager.setFilter:
# 
#  Set/add filter specified by given Id.
# ----------------------------------------------------------------------------------------------
def setFilter(self, newId, newAcquired=0, newName='', newFormat='', newContentType='', newDescription='', newRoles=[], newMetaTypes=[]):
  # Set.
  obs = getRawFilters(self)
  ob = {}
  ob['acquired'] = newAcquired
  ob['name'] = newName
  ob['format'] = newFormat
  ob['content_type'] = newContentType
  ob['description'] = newDescription
  ob['roles'] = newRoles
  ob['meta_types'] = newMetaTypes
  obs[newId] = ob
  # Set attribute.
  self.setConfProperty('ZMS.filter.filters',obs.copy())
  # Return with new id.
  return newId

# ----------------------------------------------------------------------------------------------
#  _filtermanager.delFilter:
# 
#  Delete filter specified by given Id.
# ----------------------------------------------------------------------------------------------
def delFilter(self, id):
  # Delete.
  cp = getRawFilters(self)
  obs = {}
  for key in cp.keys():
    if key != id:
      obs[key] = cp[key]
  # Set attribute.
  self.setConfProperty('ZMS.filter.filters',obs.copy())
  # Return with empty id.
  return ''


"""
################################################################################################
#
#   F I L T E R - P R O C E S S E S
#
################################################################################################
"""

# ----------------------------------------------------------------------------------------------
#  _filtermanager.setFilterProcess:
# 
#  Set/add filter-process specified by given Ids.
# ----------------------------------------------------------------------------------------------
def setFilterProcess(self, id, newProcessId, newProcessFile=None):
  # Set.
  obs = getRawFilters(self)
  ob = {}
  ob['id'] = newProcessId
  ob['file'] = newProcessFile
  pobs = obs[id].get('processes',[])
  pobs.append(ob)
  obs[id]['processes'] = pobs
  # Set attribute.
  self.setConfProperty('ZMS.filter.filters',obs.copy())
  # Return with new id.
  return len(pobs)-1

# ----------------------------------------------------------------------------------------------
#  _filtermanager.delFilterProcess:
# 
#  Delete filter-process specified by given Ids.
# ----------------------------------------------------------------------------------------------
def delFilterProcess(self, id, pid):
  # Delete.
  obs = getRawFilters(self)
  pobs = obs[id].get('processes',[])
  pobs.remove(pobs[pid])
  obs[id]['processes'] = pobs
  # Set attribute.
  self.setConfProperty('ZMS.filter.filters',obs.copy())
  # Return with empty id.
  return -1

# ----------------------------------------------------------------------------------------------
#  _filtermanager.moveFilterProcess:
# 
#  Move filter-process specified by given Ids in specified direction (-1: up, +1: down).
# ----------------------------------------------------------------------------------------------
def moveFilterProcess(self, id, pid, dir):
  # Set.
  obs = getRawFilters(self)
  pobs = obs[id].get('processes',[])
  ob = pobs[pid]
  pobs.remove(ob)
  pobs.insert(pid+dir,ob)
  obs[id]['processes'] = pobs
  # Set attribute.
  self.setConfProperty('ZMS.filter.filters',obs.copy())
  # Return with new id.
  return pid+dir


"""
################################################################################################
#
#   X S D   ( X M L - S C H E M A )
#
################################################################################################
"""

# ---------------------------------------------------------------------------------------------
#  _filtermanager.xmlSchema:
# ---------------------------------------------------------------------------------------------
def xmlSchema(self, REQUEST, RESPONSE):
  manage_lang = REQUEST.get('manage_lang')
  xsl = []
  xsl.append('<?xml version="1.0" encoding="UTF-8"?>\n')
  xsl.append('<!-- ZMS XML Schema (in work!) Generated %s -->\n'%self.getLangFmtDate(time.time(),manage_lang))
  xsl.append('<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema" elementFormDefault="qualified" attributeFormDefault="unqualified">\n')
  
  # objectTypes
  meta_types = self.dGlobalAttrs.keys()
  meta_types = self.difference_list(meta_types, ['ZMSCustom', 'ZMSTrashcan' ])
  meta_types = self.concat_list(meta_types, self.getMetaobjIds())
  for meta_type in meta_types:
    xsl.append('\t<xs:element name="%s" type="%sType">\n'%(meta_type,meta_type))
    xsl.append('\t\t<xs:annotation>\n')
    xsl.append('\t\t\t<xs:documentation>%s</xs:documentation>\n'%self.display_type(REQUEST, meta_type))
    xsl.append('\t\t</xs:annotation>\n')
    keys = self.getObjAttrs(meta_type).keys()
    keys.sort()
    xsl.append('\t\t<xs:complexContent>\n')
    if meta_type in self.dGlobalAttrs.keys():
      xsl.append('\t\t\t<xs:extension base="ZMSObject">\n')
    else:
      xsl.append('\t\t\t<xs:extension base="ZMSCustom">\n')
    for key in self.difference_list(keys, ['active', 'attr_active_start', 'attr_active_end', 'change_dt', 'change_uid', 'work_dt', 'work_uid' ] ):
      objAttr = self.getObjAttr(key)
      if objAttr['xml']:
        xsl.append('\t\t\t\t<xs:element name="%s"'%key)
        if objAttr.get('multilang'):
          xsl.append(' type="langType"')
        xsl.append('>\n')
        langStrKey = 'ATTR_%s'%key.upper()
        langStrVal = self.getLangStr(langStrKey,manage_lang)
        if langStrKey != langStrVal:
          xsl.append('\t\t\t\t\t<xs:annotation>\n')
          xsl.append('\t\t\t\t\t\t<xs:documentation>%s</xs:documentation>\n'%langStrVal)
          xsl.append('\t\t\t\t\t</xs:annotation>\n')
        xsl.append('\t\t\t\t</xs:element>\n')
    xsl.append('\t\t\t</xs:extension>\n')
    xsl.append('\t\t</xs:complexContent>\n')
    xsl.append('\t</xs:element>\n')
  # metadictTypes
  import _metadictmanager
  xsl.append(_metadictmanager.xmlSchema(self,REQUEST,RESPONSE))
  
  xsl.append('\t<!--// ############################################################ //-->\n')
  xsl.append('\t<!--// ###   Base-Objects                                       ### //-->\n')
  xsl.append('\t<!--// ############################################################ //-->\n')
  # ZMSObjectType
  xsl.append('\t<xs:complexType name="ZMSObject" mixed="true">\n')
  xsl.append('\t\t<xs:element name="active" type="langBooleanType">\n')
  xsl.append('\t\t\t<xs:annotation>\n')
  xsl.append('\t\t\t\t<xs:documentation>%s</xs:documentation>\n'%(self.getLangStr('ATTR_ACTIVE',manage_lang))) 
  xsl.append('\t\t\t</xs:annotation>\n')
  xsl.append('\t\t</xs:element>\n')
  xsl.append('\t<xs:element name="attr_active_start" type="langType">\n')
  xsl.append('\t\t\t<xs:annotation>\n')
  xsl.append('\t\t\t\t<xs:documentation>%s (%s)</xs:documentation>\n'%(self.getLangStr('ATTR_ACTIVE',manage_lang),self.getLangStr('ATTR_START_DAT',manage_lang))) 
  xsl.append('\t\t\t</xs:annotation>\n')
  xsl.append('\t\t</xs:element>\n')
  xsl.append('\t<xs:element name="attr_active_end" type="langType">\n')  
  xsl.append('\t\t\t<xs:annotation>\n')
  xsl.append('\t\t\t\t<xs:documentation>%s (%s)</xs:documentation>\n'%(self.getLangStr('ATTR_ACTIVE',manage_lang),self.getLangStr('ATTR_END_DAT',manage_lang))) 
  xsl.append('\t\t\t</xs:annotation>\n')
  xsl.append('\t\t</xs:element>\n')
  xsl.append('\t\t<xs:element name="change_dt" type="langType" />\n') 
  xsl.append('\t\t<xs:element name="change_uid" type="langType" />\n') 
  xsl.append('\t\t<xs:attribute name="id" type="xs:string" />\n')
  xsl.append('\t</xs:complexType>\n')
  # ZMSContainerObject
  xsl.append('\t<xs:complexType name="ZMSContainerObject" mixed="true">\n')
  xsl.append('\t\t<xs:complexContent>\n')
  xsl.append('\t\t\t<xs:extension base="ZMSObject">\n')
  xsl.append('\t\t\t</xs:extension>\n')
  xsl.append('\t\t</xs:complexContent>\n')
  xsl.append('\t</xs:complexType>\n')
  # ZMSCustomType
  xsl.append('\t<xs:complexType name="ZMSCustom" mixed="true">\n')
  xsl.append('\t\t<xs:complexContent>\n')
  xsl.append('\t\t\t<xs:extension base="ZMSContainerObject">\n')
  xsl.append('\t\t\t\t<xs:attribute name="meta_type">\n')
  xsl.append('\t\t\t\t\t<xs:simpleType>\n')
  xsl.append('\t\t\t\t\t\t<xs:restriction base="xs:string">\n')
  for meta_id in self.getMetaobjIds():
    xsl.append('\t\t\t\t\t\t\t<xs:enumeration value="%s"/>\n'%meta_id)
  xsl.append('\t\t\t\t\t\t</xs:restriction>\n')
  xsl.append('\t\t\t\t\t</xs:simpleType>\n')
  xsl.append('\t\t\t\t</xs:attribute>\n')
  xsl.append('\t\t\t</xs:extension>\n')
  xsl.append('\t\t</xs:complexContent>\n')
  xsl.append('\t</xs:complexType>\n')
  
  xsl.append('\t<!--// ############################################################ //-->\n')
  xsl.append('\t<!--// ###   Base-Types                                         ### //-->\n')
  xsl.append('\t<!--// ############################################################ //-->\n')
  # booleanType
  xsl.append('\t<xs:complexType base="booleanType">\n')
  xsl.append('\t\t<xs:restriction base="xs:integer">\n')
  xsl.append('\t\t\t<xs:enumeration value="0">\n')
  xsl.append('\t\t\t\t<xs:annotation>\n')
  xsl.append('\t\t\t\t\t<xs:documentation>false</xs:documentation>\n') 
  xsl.append('\t\t\t\t</xs:annotation>\n')
  xsl.append('\t\t\t</xs:enumeration>\n')
  xsl.append('\t\t\t<xs:enumeration value="1">\n')
  xsl.append('\t\t\t\t<xs:annotation>\n')
  xsl.append('\t\t\t\t\t<xs:documentation>true</xs:documentation>\n') 
  xsl.append('\t\t\t\t</xs:annotation>\n')
  xsl.append('\t\t\t</xs:enumeration>\n')
  xsl.append('\t\t</xs:restriction>\n')
  xsl.append('\t</xs:complexType>\n')
  # itemType
  xsl.append('\t<xs:complexType name="itemType" mixed="true">\n')
  xsl.append('\t\t<xs:choice minOccurs="0" maxOccurs="unbounded">\n')
  xsl.append('\t\t\t<xs:element name="list" type="listType"/>\n')
  xsl.append('\t\t\t<xs:element name="dictionary" type="dictionaryType"/>\n')
  xsl.append('\t\t</xs:choice>\n')
  xsl.append('\t\t<xs:attribute name="key" />\n')
  xsl.append('\t</xs:complexType>\n')
  # dataType
  xsl.append('\t<xs:complexType name="dataType">\n')
  xsl.append('\t\t<xs:attribute name="width" type="xs:short"/>\n')
  xsl.append('\t\t<xs:attribute name="height" type="xs:short"/>\n')
  xsl.append('\t\t<xs:attribute name="content_type" type="xs:string" use="required"/>\n')
  xsl.append('\t\t<xs:attribute name="filename" type="xs:string" use="required"/>\n')
  xsl.append('\t</xs:complexType>\n')
  # listType
  xsl.append('\t<xs:complexType name="listType" content="elementOnly">\n')
  xsl.append('\t\t<xs:choice minOccurs="0" maxOccurs="unbounded">\n')
  xsl.append('\t\t\t<xs:element name="item" type="itemType"/>\n')
  xsl.append('\t\t</xs:choice>\n')
  xsl.append('\t</xs:complexType>\n')
  # dictionaryType
  xsl.append('\t<xs:complexType name="dictionaryType" content="elementOnly">\n')
  xsl.append('\t\t<xs:sequence>\n')
  xsl.append('\t\t\t<xs:element name="item" type="itemType" maxOccurs="unbounded"/>\n')
  xsl.append('\t\t</xs:sequence>\n')
  xsl.append('\t</xs:complexType>\n')

  xsl.append('\t<!--// ############################################################ //-->\n')
  xsl.append('\t<!--// ###   Base-Multilanguage                                 ### //-->\n')
  xsl.append('\t<!--// ############################################################ //-->\n')
  # langType
  xsl.append('\t<xs:complexType name="langType" mixed="true">\n')
  xsl.append('\t\t<xs:element name="lang">\n')
  xsl.append('\t\t\t<xs:attribute name="id" use="required">\n')
  xsl.append('\t\t\t\t<xs:simpleType>\n')
  xsl.append('\t\t\t\t\t<xs:restriction base="xs:string">\n')
  for langId in self.getLangIds():
    xsl.append('\t\t\t\t\t\t<xs:enumeration value="%s">\n'%langId)
    xsl.append('\t\t\t<xs:annotation>\n')
    xsl.append('\t\t\t\t<xs:documentation>%s</xs:documentation>\n'%(self.getLanguageLabel(langId))) 
    xsl.append('\t\t\t</xs:annotation>\n')
    xsl.append('\t\t\t\t\t\t</xs:enumeration>\n')
  xsl.append('\t\t\t\t\t</xs:restriction>\n')
  xsl.append('\t\t\t\t</xs:simpleType>\n')
  xsl.append('\t\t</xs:attribute>\n')
  xsl.append('\t\t</xs:element>\n')
  xsl.append('\t</xs:complexType>\n')
  # langBooleanType
  xsl.append('\t<xs:complexType name="langBooleanType" mixed="true">\n')
  xsl.append('\t\t<xs:extension base="langType">\n')
  xsl.append('\t\t\t<xs:restriction base="booleanType" />\n')
  xsl.append('\t\t</xs:extension>\n')
  xsl.append('\t</xs:complexType>\n')
  # langDataType
  xsl.append('\t<xs:complexType name="langDataType" content="elementOnly">\n')
  xsl.append('\t\t<xs:extension base="langType">\n')
  xsl.append('\t\t\t<xs:element name="data" type="dataType" minOccurs="0" maxOccurs="1"/>\n')
  xsl.append('\t\t</xs:extension>\n')
  xsl.append('\t</xs:complexType>\n')
  # langListType
  xsl.append('\t<xs:complexType name="langListType" content="elementOnly">\n')
  xsl.append('\t\t<xs:extension base="langType">\n')
  xsl.append('\t\t\t<xs:element name="list" type="listType" minOccurs="0" maxOccurs="1"/>\n')
  xsl.append('\t\t</xs:extension>\n')
  xsl.append('\t</xs:complexType>\n')
  # langDictionaryType
  xsl.append('\t<xs:complexType name="langDictionaryType" content="elementOnly">\n')
  xsl.append('\t\t<xs:extension base="langType">\n')
  xsl.append('\t\t\t<xs:element name="dictionary" type="dictionaryType" minOccurs="0" maxOccurs="1"/>\n')
  xsl.append('\t\t</xs:extension>\n')
  xsl.append('\t</xs:complexType>\n')
  
  #
  xsl.append('</xs:schema>\n')
  ext = 'xml'
  content_type = 'text/'+ext
  filename = 'zmsSchema.'+ext
  RESPONSE.setHeader('Content-Type',content_type)
  RESPONSE.setHeader('Content-Disposition','inline;filename=%s'%filename)
  return ''.join(xsl)


"""
################################################################################################
#
#   I M / E X P O R T
#
################################################################################################
"""

# ---------------------------------------------------------------------------------------------
#  _filtermanager.printableHtml:
# ---------------------------------------------------------------------------------------------
def printableHtml(self, sectionize, REQUEST):
  html = ''
  html += '<html>\n'
  html += '<head>\n'
  html += '<title>%s</title>\n'%(self.getTitle(REQUEST))
  html += '<link rel="schema.DC" href="http://purl.org/DC/elements/1.1/" title="http://purl.org/DC/elements/1.1/">\n'
  html += '<link rel="stylesheet" type="text/css" href="%s/f_css_printhtml">\n'%self.getDocumentElement().absolute_url()
  html += '</head>\n'
  html += '<body>\n'
  html += self.printHtml(0, sectionize, _globals.MySectionizer(), REQUEST )
  html += '</body>\n'
  html += '</html>\n'
  return html


# ---------------------------------------------------------------------------------------------
#  _filtermanager.processData:
#
#  Process data with custom transformation.
# ---------------------------------------------------------------------------------------------
def processData(self, processId, data, trans=None):
  # Create temporary folder.
  folder = tempfile.mktemp()
  # Save data to file.
  filename = _fileutil.getOSPath('%s/in.dat'%folder)
  _fileutil.exportObj(data, filename)
  # Process file.
  filename = processFile(self, processId, filename, trans, folder)
  # Read data from file.
  f = open(filename, 'rb')
  data = f.read()
  f.close()
  # Remove temporary folder.
  _fileutil.remove(folder, deep=1)
  # Return data.
  return data


# ---------------------------------------------------------------------------------------------
#  _filtermanager.processFile:
#
#  Process file with custom transformation.
# ---------------------------------------------------------------------------------------------
def processFile(self, processId, filename, trans=None):
  folder = _fileutil.getFilePath(filename)
  processOb = self.getProcess(processId)
  command = processOb.get('command')
  # Save transformation to file.
  if trans is not None:
    transfilename = '%s/trans.dat'%folder
    f = open(transfilename,'w')
    f.write(trans)
    f.close()
    command = command.replace('{trans}',transfilename)
  # Execute command.
  filename = processCommand(self, filename, command)
  # Return filename.
  return filename


# ---------------------------------------------------------------------------------------------
#  _filtermanager.processCommand:
#
#  Process file with command.
# ---------------------------------------------------------------------------------------------
def processCommand(self, filename, command):
  infilename = filename
  outfilename = filename + '.tmp'
  if command.find('{out}') < 0:
    outfilename = infilename
  command = command.replace('{in}',infilename)
  command = command.replace('{out}',outfilename)
  os.system(command)
  # Return filename.
  return outfilename


# ---------------------------------------------------------------------------------------------
#  _filtermanager.processFilter:
# ---------------------------------------------------------------------------------------------
def processFilter(self, ob_filter, folder, filename):
  for process in ob_filter.get('processes',[]):
    processId = process.get('id')
    trans = process.get('file', None)
    print processId, trans
    if trans is not None and len(str(trans)) > 0:
      trans = trans.getData()
    filename = processFile(self, processId, filename, trans)
  # Return filename.
  return filename


# ---------------------------------------------------------------------------------------------
#  _filtermanager.importFilter:
# ---------------------------------------------------------------------------------------------
def importFilter(self, filename, id, REQUEST):
  ob_filter = self.getFilter(id)
  folder = _fileutil.getFilePath(filename)
  # Process filter.
  filename = processFilter(self, ob_filter, folder, filename)
  # Return filename.
  return filename


# ---------------------------------------------------------------------------------------------
#  _filtermanager.exportFilter:
# ---------------------------------------------------------------------------------------------
def exportFilter(self, id, zip_result, REQUEST):
  from _exportable import localHtml, localPrintHtml
  ob_filter = self.getFilter(id)
  # Create temporary folder.
  folder = self.__prepare_temp_export_folder__(REQUEST)
  if ob_filter.get('format','') == 'XHTML':
    # Set XHTML.
    filename = _fileutil.getOSPath('%s/export.html'%folder)
    html = printableHtml(self, 0, REQUEST )
    html = localHtml(self, self, html, REQUEST )
    html = localPrintHtml(self, self, html, REQUEST )
    _fileutil.exportObj(html, filename)
  elif ob_filter.get('format','') == 'XML':
    # Set XML.
    data = self.toXml(REQUEST)
    filename = _fileutil.getOSPath('%s/export.xml'%folder)
    f = open(filename, 'w')
    f.write(data)
    f.close()
  else:
    raise "Unkown formmat '%s'"%ob_filter.get('format','')
  # Process filter.
  filename = processFilter(self, ob_filter, folder, filename)
  # Zip File.
  if zip_result:
    if ob_filter.get('content_type','content/unknown').find('text/') > 0:
      zipfiles = _fileutil.getOSPath('%s/*.*'%folder)
    else:
      zipfiles = filename
    filename = 'exportFilter.zip'
    data = _fileutil.buildZipArchive(zipfiles)
    content_type = 'application/zip'
  # Read File.
  else:
    content_type = ob_filter.get('content_type','content/unknown')
    f = open(filename, 'r')
    data = f.read()
    f.close()
    filename = 'exportFilter.%s'%content_type[content_type.find('/')+1:]
  # Remove temporary folder.
  _fileutil.remove(folder, deep=1)
  # Return.
  return filename, data, content_type


################################################################################################
################################################################################################
###
###   class FilterManager
###
################################################################################################
################################################################################################
class FilterManager: 

    # Management Interface.
    # ---------------------
    manage_customizeFilterForm = HTMLFile('dtml/zms/manage_customizefilterform', globals())


    # ------------------------------------------------------------------------------------------
    #  FilterManager.getProcessIds:
    # 
    #  Returns list of process-Ids.
    # ------------------------------------------------------------------------------------------
    def getProcessIds(self, sort=1):
      obs = getRawProcesses(self)
      ids = obs.keys()
      if self.getConfProperty('Portal.Master'):
        masterHome = getattr(self.getHome(),self.getConfProperty('Portal.Master'))
        masterDocElmnt = masterHome.objectValues(['ZMS'])[0]
        ids.extend(masterDocElmnt.getProcessIds())
      if sort:
        mapping = map(lambda x: (self.getProcess(x)['name'],x),ids)
        mapping.sort()
        ids = map(lambda x: x[1],mapping)
      return ids

    # ------------------------------------------------------------------------------------------
    #  FilterManager.getProcess:
    # 
    #  Returns process specified by Id.
    # ------------------------------------------------------------------------------------------
    def getProcess(self, id):
      ob = None
      if self.getConfProperty('Portal.Master'):
        masterHome = getattr(self.getHome(),self.getConfProperty('Portal.Master'))
        masterDocElmnt = masterHome.objectValues(['ZMS'])[0]
        if id in masterDocElmnt.getProcessIds():
          ob = masterDocElmnt.getProcess(id)
          ob['acquired'] = 1
      if ob is None:
        obs = getRawProcesses(self)
        ob = obs[id].copy()
      ob['id'] = id
      return ob


    # ------------------------------------------------------------------------------------------
    #  FilterManager.getFilterIds:
    # 
    #  Returns list of filter-Ids.
    # ------------------------------------------------------------------------------------------
    def getFilterIds(self, sort=1):
      obs = getRawFilters(self)
      ids = obs.keys()
      if sort:
        mapping = map(lambda x: (self.getFilter(x)['name'],x),ids)
        mapping.sort()
        ids = map(lambda x: x[1],mapping)
      return ids

    # ------------------------------------------------------------------------------------------
    #  FilterManager.getFilter:
    # 
    #  Returns filter specified by Id.
    # ------------------------------------------------------------------------------------------
    def getFilter(self, id):
      ob = getRawFilters(self)[id].copy()
      ob['id'] = id
      # Acquire from parent.
      if ob.get('acquired',0):
        masterHome = getattr(self.getHome(),self.getConfProperty('Portal.Master'))
        masterDocElmnt = masterHome.objectValues(['ZMS'])[0]
        ob = masterDocElmnt.getFilter(id)
        ob['acquired'] = 1
      return ob

    # ------------------------------------------------------------------------------------------
    #  FilterManager.getFilterProcesses:
    # 
    #  Returns list of processes for filter specified by Id.
    # ------------------------------------------------------------------------------------------
    def getFilterProcesses(self, id):
      ob = self.getFilter(id).get('processes',[])
      return ob


    ###############################################################################################
    #  FilterManager.manage_changeFilter:
    #
    #  Customize filter.
    ###############################################################################################
    def manage_changeFilter(self, lang, manage_lang, btn='', key='', REQUEST=None, RESPONSE=None):
      """ FilterManager.manage_changeFilter """
      message = ''
      id = REQUEST.get('id','')
      pid = REQUEST.get('pid',-1)

      # Acquire.
      # --------
      if btn == self.getLangStr('BTN_ACQUIRE',manage_lang):
        newId = REQUEST.get('aq_id')
        newAcquired = 1
        id = setFilter(self, newId, newAcquired)
        message = self.getLangStr('MSG_INSERTED',manage_lang)%id

      # Change.
      # -------
      elif btn == self.getLangStr('BTN_CHANGE',manage_lang):
	cp = self.getFilter(id)
	# Filter.
        newId = REQUEST.get('inpId').strip()
        newAcquired = 0
        newName = REQUEST.get('inpName').strip()
        newFormat = REQUEST.get('inpFormat').strip()
        newContentType = REQUEST.get('inpContentType').strip()
	newDescription = REQUEST.get('inpDescription').strip()
	newRoles = REQUEST.get('inpRoles',[])
	newMetaTypes = REQUEST.get('inpMetaTypes',[])
        id = delFilter(self, id)
        id = setFilter(self, newId, newAcquired, newName, newFormat, newContentType, newDescription, newRoles, newMetaTypes)
	# Filter Processes.
	c = 0
	for filterProcess in cp.get('processes',[]):
          newProcessId = REQUEST.get('newFilterProcessId_%i'%c)
          newProcessFile = REQUEST.get('newFilterProcessFile_%i'%c)
          if getattr(newProcessFile, '__class__',None) is ZPublisher.HTTPRequest.FileUpload:
            if len(getattr(newProcessFile, 'filename',''))==0:
              newProcessFile = filterProcess.get('file', None)
            else:
              newProcessFile = _blobfields.createBlobField(self, _globals.DT_FILE, newProcessFile)
          setFilterProcess(self, id, newProcessId, newProcessFile)
          c += 1
        message = self.getLangStr('MSG_CHANGED',manage_lang)

      # Delete.
      # -------
      elif btn == self.getLangStr('BTN_DELETE',manage_lang) and key == 'obj':
        id = delFilter(self, id)
        message = self.getLangStr('MSG_DELETED',manage_lang)%int(1)
      elif btn == 'delete' and key == 'attr':
        pid = delFilterProcess(self, id, pid)
        message = self.getLangStr('MSG_DELETED',manage_lang)%int(1)

      # Export.
      # -------
      elif btn == self.getLangStr('BTN_EXPORT',manage_lang):
        return exportXml(self, REQUEST, RESPONSE)

      # Import.
      # -------
      elif btn == self.getLangStr('BTN_IMPORT',manage_lang):
        f = REQUEST['file']
        importXml(self, xml=f)
        message = self.getLangStr('MSG_IMPORTED',manage_lang)%('<i>%s</i>'%f.filename)

      # Init.
      # -----
      elif btn == self.getLangStr('BTN_INIT',manage_lang):
	filename = initConf(self, REQUEST)
        message = self.getLangStr('MSG_IMPORTED',manage_lang)%('<i>%s</i>'%filename)

      # Insert.
      # -------
      elif btn == self.getLangStr('BTN_INSERT',manage_lang):
        if key == 'obj':
          newId = REQUEST.get('newId').strip()
          newAcquired = 0
          newName = REQUEST.get('newName').strip()
          newFormat = REQUEST.get('newFormat').strip()
          newContentType = REQUEST.get('newContentType').strip()
          id = setFilter(self, newId, newAcquired, newName, newFormat, newContentType)
          message = self.getLangStr('MSG_INSERTED',manage_lang)%id
        elif key == 'attr':
          newProcessId = REQUEST.get('newFilterProcessId')
          newProcessFile = REQUEST.get('newFilterProcessFile')
          if getattr(newProcessFile, '__class__',None) is ZPublisher.HTTPRequest.FileUpload:
            if len(getattr(newProcessFile, 'filename',''))==0:
              newProcessFile = None
            else:
              newProcessFile = _blobfields.createBlobField(self, _globals.DT_FILE, newProcessFile)
          pid = setFilterProcess(self, id, newProcessId, newProcessFile)
          message = self.getLangStr('MSG_INSERTED',manage_lang)%newProcessId

      # Move down.
      # --------
      elif btn == 'move_down':
        pid = moveFilterProcess(self, id, pid, +1)
        message = self.getLangStr('MSG_MOVEDOBJDOWN', manage_lang)%("<i>%s</i>"%pid)

      # Move up.
      # --------
      elif btn == 'move_up':
        pid = moveFilterProcess(self, id, pid, -1)
        message = self.getLangStr('MSG_MOVEDOBJUP', manage_lang)%("<i>%s</i>"%pid)

      # XSD.
      # ----
      elif btn == 'xsd':
        return xmlSchema(self, REQUEST, RESPONSE)

      # Return with message.
      message = urllib.quote(message)
      return RESPONSE.redirect('manage_customizeFilterForm?id=%s&pid:int=%i&lang=%s&manage_lang=%s&manage_tabs_message=%s'%(id,pid,lang,manage_lang,message))


    ###############################################################################################
    #  FilterManager.manage_changeProcess:
    #
    #  Customize process.
    ###############################################################################################
    def manage_changeProcess(self, lang, manage_lang, btn='', key='', REQUEST=None, RESPONSE=None):
      """ FilterManager.manage_changeProcess """
      message = ''
      id = REQUEST.get('id','')

      # Change.
      # -------
      if btn == self.getLangStr('BTN_CHANGE',manage_lang):
        newId = REQUEST.get('inpId').strip()
        newAcquired = 0
        newName = REQUEST.get('inpName').strip()
        newCommand = REQUEST.get('inpCommand').strip()
        id = delProcess(self, id)
        id = setProcess(self, newId, newAcquired, newName, newCommand)
        message = self.getLangStr('MSG_CHANGED',manage_lang)

      # Delete.
      # -------
      elif btn == self.getLangStr('BTN_DELETE',manage_lang):
        id = delProcess(self, id)
        message = self.getLangStr('MSG_DELETED',manage_lang)%int(1)

      # Export.
      # -------
      elif btn == self.getLangStr('BTN_EXPORT',manage_lang):
        return exportXml(self, REQUEST, RESPONSE)

      # Import.
      # -------
      elif btn == self.getLangStr('BTN_IMPORT',manage_lang):
        f = REQUEST['file']
        importXml(self, xml=f)
        message = self.getLangStr('MSG_IMPORTED',manage_lang)%('<i>%s</i>'%f.filename)

      # Init.
      # -----
      elif btn == self.getLangStr('BTN_INIT',manage_lang):
	filename = initConf(self, REQUEST)
        message = self.getLangStr('MSG_IMPORTED',manage_lang)%('<i>%s</i>'%filename)

      # Insert.
      # -------
      elif btn == self.getLangStr('BTN_INSERT',manage_lang):
        newId = REQUEST.get('newId').strip()
        newAcquired = 0
        newName = REQUEST.get('newName').strip()
        newCommand = REQUEST.get('newCommand').strip()
        id = setProcess(self, newId, newAcquired, newName, newCommand)
        message = self.getLangStr('MSG_INSERTED',manage_lang)%id

      # XSD.
      # ----
      elif btn == 'xsd':
        return xmlSchema(self, REQUEST, RESPONSE)

      # Return with message.
      message = urllib.quote(message)
      return RESPONSE.redirect('manage_customizeFilterForm?id=%s&lang=%s&manage_lang=%s&manage_tabs_message=%s'%(id,lang,manage_lang,message))

###################################################################################################
