###################################################################################################
# _pathhandler.py
#
# $Id: _pathhandler.py,v 1.2 2003/10/10 18:33:26 dnordmann Exp $
# $Name:  $
# $Author: dnordmann $
# $Revision: 1.2 $
#
# Implementation of class PathHandler (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
import copy
# Product Imports.
import zmscontainerobject


###################################################################################################
###################################################################################################
###
###   class PathHandler:
###
###   Based on the Zope-Product PathHandler (http://www.zope.org/Members/NIP/PathHandler).
###
###################################################################################################
###################################################################################################
class PathHandler: 
        
    # ------------------------------------------------------------------------------------------
    #  PathHandler.__bobo_traverse__
    # ------------------------------------------------------------------------------------------
    def __bobo_traverse__(self, TraversalRequest, name):
      # If this is the first time this __bob_traverse__ method has been called
      # in handling this traversal request, store the path_to_handle
      
      if not TraversalRequest.has_key('path_to_handle'):
          # Make a reversed copy of the TraversalRequestNameStack
          TraversalRequestNameStackReversed=copy.copy(TraversalRequest['TraversalRequestNameStack'])
          TraversalRequestNameStackReversed.reverse()
          
          # Set path_to_handle in the TraversalRequest.
          TraversalRequest['path_to_handle']=[name]+TraversalRequestNameStackReversed
      
      # If the name is in the list of attributes, call it.
      if getattr(self,name,None) is not None:
        return getattr(self,name)
      
      # otherwise do some 'magic'
      else:
        # If there's no more names left to handle, return the path handling method
        # to the traversal machinery so it gets called next
        if len(TraversalRequest['TraversalRequestNameStack'])==0:
          return self.pathhandler
        # If there are more names to handle, return ourself, so this __bobo_traverse__ gets
        # called again and we get to ignore the other names.
        else:
          return self


    # ------------------------------------------------------------------------------------------
    #  PathHandler.__call__
    # ------------------------------------------------------------------------------------------
    def __call__(self, REQUEST={}, **kw):
      """"""
      # This handles the special case where the Path Handler's id is the last one in the
      # traversal request.
      # eg: when the URL ends with the id of the Path Handler.
      # > REQUEST['path_to_handle']=[]
      # > return self.pathhandler(REQUEST)
      return self


    # ------------------------------------------------------------------------------------------
    #  PathHandler.pathob
    # ------------------------------------------------------------------------------------------
    def pathob(self, path_to_handle, REQUEST):
      if self.getConfProperty('ZMS.pathhandler',0)==1:
        langs = self.getLangIds()
        path_ob = self.getDocumentElement()
        path_index = 0
        while 1:
          if path_index == len(path_to_handle):
            if path_ob.hasAccess(REQUEST):
              return path_ob
            else:
              return None
          path_item = path_to_handle[path_index].lower()
          if path_index == 0 and path_item in langs:
            REQUEST.set('lang',path_item) 
          else:
            obs = path_ob.getChildNodes(REQUEST,self.PAGES)
            path_ob = None
            for ob in obs:
              id = ob.id
              titlealt = ob.getObjProperty('attr_dc_identifier_url_node',REQUEST)
              if len(titlealt)==0:
                titlealt = ob.getTitlealt(REQUEST)
              if path_item == id or path_item == self.id_quote(titlealt):
                path_ob = ob
            if path_ob is None: 
              break
          path_index = path_index + 1 
      return None


    # ------------------------------------------------------------------------------------------
    #  PathHandler.pathhandler
    # ------------------------------------------------------------------------------------------
    def pathhandler(self, REQUEST, RESPONSE=None):
      """ PathHandler.pathhandler """
      path_to_handle = REQUEST.get('path_to_handle',[])
      while '/' in path_to_handle:
        path_to_handle = path_to_handle[path_to_handle.index('/')+1:]
      path_ob = self.pathob(path_to_handle,REQUEST)
      if path_ob is not None:
        path_ob = zmscontainerobject.getPageWithElements(path_ob,REQUEST)
        return self.index_html(path_ob,REQUEST)
      else:
        raise "Resource not found",'<h2>Site-Error</h2><b>Sorry, there is no web page matching your request.</b> It is possible you typed the address incorrectly, or that the page no longer exists.<hr><b>Resource<b> '+''.join(map(lambda x: x+'/',path_to_handle))+' GET'

###################################################################################################
