###################################################################################################
# _versionmanager.py
#
# $Id: _versionmanager.py,v 1.4 2003/10/10 18:33:26 dnordmann Exp $
# $Name:  $
# $Author: dnordmann $
# $Revision: 1.4 $
#
# Implementation of class VersionManager (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from __future__ import nested_scopes
from Globals import HTMLFile
import copy
import time
import urllib
# Product Imports.
import _accessmanager
import _globals 
import _blobfields 
import _zmsattributecontainer


# DEPRECATED: Products.zms._myobjects.MyFile
class MyFile(_blobfields.MyFile): pass
# DEPRECATED: Products.zms._myobjects.MyImage
class MyImage(_blobfields.MyImage): pass


# ---------------------------------------------------------------------------------------------
#  _versionmanager.getObjStates:
#
#  Returns a list of object-states (including language suffixes).
# ---------------------------------------------------------------------------------------------
def getObjStates(self):
  if not hasattr(self,'__work_state__'):
    self.__work_state__ = _globals.MyClass()
  return getattr(self.__work_state__,'states',[])

# ---------------------------------------------------------------------------------------------
#  _versionmanager.setObjState:
#
#  Sets object-state.
# ---------------------------------------------------------------------------------------------
def setObjState(self, obj_state, lang):
  state = getObjStateName(obj_state,lang)
  states = getObjStates(self)
  if not state in states:
    states.append(state)
  self.__work_state__.states = copy.deepcopy(states)
  self.__work_state__ = copy.deepcopy(self.__work_state__)

# ---------------------------------------------------------------------------------------------
#  _versionmanager.delObjStates:
#
#  Deletes object-state of given object.
# ---------------------------------------------------------------------------------------------
def delObjStates(self, obj_states=[], REQUEST={}):
  lang = REQUEST.get('lang',self.getPrimaryLanguage())
  states = getObjStates(self)
  for obj_state in obj_states:
    while obj_state in states:
      del states[states.index(obj_state)]
    while getObjStateName(obj_state,lang) in states:
      del states[states.index(getObjStateName(obj_state,lang))]
  self.__work_state__.states = copy.deepcopy(states)
  self.__work_state__ = copy.deepcopy(self.__work_state__)


# ---------------------------------------------------------------------------------------------
#  _versionmanager.setWorkflowEventBy:
#
#  Applies information about user-id and date of workflow-event.
# ---------------------------------------------------------------------------------------------
def setWorkflowEventBy(self, REQUEST):
  # Set Properties.
  lang = REQUEST.get('lang',self.getPrimaryLanguage())
  auth_user = REQUEST.get('AUTHENTICATED_USER','Anonymous User')
  self.setObjProperty('work_uid',str(auth_user),lang)
  self.setObjProperty('work_dt',time.time(),lang)


# ---------------------------------------------------------------------------------------------
#  _versionmanager.setChangedBy:
#
#  Applies information about user-id and date of change.
# ---------------------------------------------------------------------------------------------
def setChangedBy(self, REQUEST, reset=1):
  # Add Version.
  if reset and self.getConfProperty('ZMS.Version.active',0)==1:
    req = {'lang':'*'}
    workAttrCntnr = _zmsattributecontainer.manage_addZMSAttributeContainer(self)
    self.cloneObjAttrs(getattr(self,self.version_work_id),workAttrCntnr,req)
    self.version_work_id = workAttrCntnr.id
  # Set Properties.
  lang = REQUEST.get('lang',self.getPrimaryLanguage())
  auth_user = REQUEST.get('AUTHENTICATED_USER','Anonymous User')
  self.setObjProperty('change_lang',lang)
  self.setObjProperty('change_action',REQUEST.get('URL','/').split('/')[-1])
  self.setObjProperty('change_uid',str(auth_user),lang)
  self.setObjProperty('change_dt',time.localtime(time.time()),lang)


# ---------------------------------------------------------------------------------------------
#  _versionmanager.getObjStateName:
#
#  Returns name object state in given language.
# ---------------------------------------------------------------------------------------------
def getObjStateName(obj_state, lang):
  obj_state_name = obj_state
  if lang is not None:
    obj_state_name += '_'+lang.upper()
  return obj_state_name


# ---------------------------------------------------------------------------------------------
#  _versionmanager.pack:
#
#  Pack object and process children.
# ---------------------------------------------------------------------------------------------
def pack(self): 
  now = time.time()
  #-- Get days.
  days = self.getConfProperty('ZMS.Version.autopack','')
  try: days = int(days)
  except: days = -1
  #-- Get IDs.
  ids = []
  for ob in self.getObjVersions():
    delete = 1
    for lang in self.getLangIds():
      req = {'lang':lang,'preview':'preview'}
      change_dt = ob.getObjProperty('change_dt',req)
      if change_dt is not None:
        delete = delete and _globals.daysBetween(change_dt,now)>days
    delete = delete and ob.id not in [self.version_live_id,self.version_work_id]
    if delete:
      ids.append(ob.id)
  #-- Delete objects.
  self.manage_delObjects(ids=ids)
  #-- Process child-objects.
  for ob in self.getChildNodes():
    pack(ob)


# ---------------------------------------------------------------------------------------------
#  _versionmanager.doAutocommit:
#
#  Commit pending changes of all objects.
# ---------------------------------------------------------------------------------------------
def doAutocommit(self, REQUEST): 
  #++ if _globals.debug: print "[%s.doAutocommit]"%self.meta_type
  
  ##### Auto-Commit ####
  lang = REQUEST['lang']
  for s_lang in self.getLangIds():
    REQUEST.set('lang',s_lang)
    self.onChangeObj(REQUEST,forced=1)
  REQUEST.set('lang',lang)
  
  ##### Process child-objects ####
  for ob in self.getChildNodes():
    doAutocommit(ob,REQUEST)


###################################################################################################
###################################################################################################
###
###   class VersionManager
###
###################################################################################################
###################################################################################################
class VersionManager: 

    # Management Interface.
    # ---------------------
    version_object_state = HTMLFile('dtml/versionmanager/version_object_state', globals())


    # ---------------------------------------------------------------------------------------------
    #  VersionManager.isObjModified:
    # ---------------------------------------------------------------------------------------------
    def isObjModified(self, REQUEST):
      return self.inObjStates(['STATE_NEW','STATE_MODIFIED','STATE_DELETED'],REQUEST)


    # ---------------------------------------------------------------------------------------------
    #  VersionManager.hasObjModifiedChildren:
    # ---------------------------------------------------------------------------------------------
    def hasObjModifiedChildren(self, REQUEST):
      state = 0
      for child in self.getObjChildren('',REQUEST):
        state = state + child.isObjModified(REQUEST)
      return state


    # ---------------------------------------------------------------------------------------------
    #  VersionManager.run_pack_history:
    #
    #  Runs packing of history.
    # ---------------------------------------------------------------------------------------------
    def run_pack_history(self, forced=0):
      now = time.time()
      last_run = self.getConfProperty('ZMS.Version.autopack.datetime',None)
      if forced or \
         last_run is None or \
         _globals.daysBetween(last_run,now)>1:
        #-- Pack objects.
        if self.getConfProperty('ZMS.Version.active',0)==1:
          pack(self)
        #-- Update time-stamp.
        self.setConfProperty('ZMS.Version.autopack.datetime',time.localtime(time.time()))


    # ---------------------------------------------------------------------------------------------
    #  VersionManager.resetObjTranslation:
    #
    #  Resets translation.
    # ---------------------------------------------------------------------------------------------
    def resetObjTranslation(self):
      for ob in self.getObjVersions():
        for lang in self.getLangIds():
          if lang != self.getPrimaryLanguage():
            setattr(ob,'change_uid_%s'%lang,'')


    # ---------------------------------------------------------------------------------------------
    #  VersionManager.resetObjVersion:
    #
    #  Resets work-version of object-attributes.
    # ---------------------------------------------------------------------------------------------
    def resetObjVersion(self):
      # States.
      self.__work_state__ = _globals.MyClass()
      # Versions.
      if len(self.getObjVersions())==0:
        req = {'lang':'*'}
        liveAttrCntnr = _zmsattributecontainer.manage_addZMSAttributeContainer(self)
        self.cloneObjAttrs(self,liveAttrCntnr,req)
        self.version_live_id = liveAttrCntnr.id
        workAttrCntnr = _zmsattributecontainer.manage_addZMSAttributeContainer(self)
        self.cloneObjAttrs(liveAttrCntnr,workAttrCntnr,req)
        self.version_work_id = workAttrCntnr.id


    # ---------------------------------------------------------------------------------------------
    #  VersionManager.getObjVersion:
    #
    #  Live or work-version of object-attributes.
    # ---------------------------------------------------------------------------------------------
    def getObjVersion(self, REQUEST={}):
      ob = None
      id = None
      if REQUEST.get('ZMS_VERSION',None) is not None:
        if REQUEST['ZMS_VERSION'].find(self.id+'/')==0:
          id = REQUEST['ZMS_VERSION'][len(self.id+'/'):]
      if id is None:
        if _globals.isPreviewRequest(REQUEST):
          id = self.version_work_id
        else:
          id = self.version_live_id
      ob = getattr(self,id)
      s = ob.id # Never delete this line!
      return ob


    # ---------------------------------------------------------------------------------------------
    #  VersionManager.getObjVersions:
    # ---------------------------------------------------------------------------------------------
    def getObjVersions(self, REQUEST={}):
      obs = []
      for ob in self.objectValues(['ZMSAttributeContainer']):
        lang = REQUEST.get('lang','*')
        change_lang = ob.getObjProperty('change_lang',REQUEST)
        if (lang=='*') or \
           (lang!='*' and ob.id!=self.version_live_id and \
            ((len(obs)==0) or \
             (change_lang==lang) or \
             (change_lang=='' and lang==self.getPrimaryLanguage()))):
          obs.insert(0,ob)
      return obs


    # ---------------------------------------------------------------------------------------------
    #  VersionManager.isCommitted:
    #
    #  Checks if object can by displayed.
    # ---------------------------------------------------------------------------------------------
    def isCommitted(self, REQUEST):
      lang = REQUEST.get('lang',self.getPrimaryLanguage())
      preview = REQUEST.get('preview','') == 'preview'
      states = getObjStates(self)
      committed = 0
      committed = committed or (not preview and not getObjStateName('STATE_NEW',lang) in states)
      committed = committed or (preview and not getObjStateName('STATE_DELETED',lang) in states)
      return committed


    """
    ###############################################################################################
    #
    #  Object State
    #
    ###############################################################################################
    """

    # ---------------------------------------------------------------------------------------------
    #  VersionManager.resetObjStates:
    #
    #  Resets object-state of this object.
    # ---------------------------------------------------------------------------------------------
    def resetObjStates(self, REQUEST):
      delObjStates(self,['STATE_NEW','STATE_MODIFIED','STATE_MODIFIED_SILENT','STATE_DELETED','STATE_COMMITTED','STATE_REQUESTED','STATE_ROLLEDBACK','STATE_REJECTED'],REQUEST)


    # ---------------------------------------------------------------------------------------------
    #  VersionManager.inObjStates:
    #
    #  Checks if given states are in object-states of this object.
    # ---------------------------------------------------------------------------------------------
    def inObjStates(self, obj_states, REQUEST):
      states = getObjStates(self)
      states.extend(self.getObjStateNames(REQUEST))
      if len(states)>0:
        lang = REQUEST.get('lang',self.getPrimaryLanguage())
        for obj_state in obj_states:
          obj_state_name = getObjStateName(obj_state,lang)
          if obj_state_name in states:
            return 1
      return 0


    # ---------------------------------------------------------------------------------------------
    #  VersionManager.getObjStateNames:
    #
    #  Returns a list of normalized object-states (language suffixes stripped off).
    # ---------------------------------------------------------------------------------------------
    def getObjStateNames(self, REQUEST={}):
      states = getObjStates(self)
      
      lang = REQUEST.get('lang',None)
      if lang is None:
        obj_states = states
      else:
        obj_states = []
        for obj_state in ['STATE_NEW','STATE_MODIFIED','STATE_MODIFIED_SILENT','STATE_DELETED']:
          obj_state_name = getObjStateName(obj_state,lang)
          if obj_state_name in states:
            obj_states.append(obj_state_name)
      
      # Special Objects / Link-Containers.
      children = []
      if self.meta_type == 'ZMSCustom' and not self.isPage():
        children = self.getObjChildren('',REQUEST)
      elif self.meta_type == 'ZMSLinkContainer':
        children = self.getChildNodes(REQUEST,['ZMSLinkElement'])
      obj_state = 'STATE_MODIFIED'
      for child in children:
        for state in child.getObjStateNames(REQUEST):
          lang = state[state.rfind('_')+1:].lower()
          obj_state_name = getObjStateName(obj_state,lang)
          if not obj_state_name in obj_states:
            obj_states.append(obj_state_name)
      
      # Return Object State Names.
      return obj_states


    # ---------------------------------------------------------------------------------------------
    #  VersionManager.isOnlyModifiedSilent:
    #
    #  Returns true if object is only modified silent.
    # ---------------------------------------------------------------------------------------------
    def isOnlyModifiedSilent(self):
      silent = 1
      for state in getObjStates(self):
        silent = silent and \
                 (state.find('STATE_MODIFIED_SILENT') == 0 or
                  state.find('STATE_COMMITTED') == 0 or
                  state.find('STATE_REQUESTED') == 0 or
                  state.find('STATE_REJECTED') == 0 or 
                  state.find('STATE_ROLLEDBACK') == 0)
      return silent

    # ---------------------------------------------------------------------------------------------
    #  VersionManager.setObjStateModifiedSilent
    # ---------------------------------------------------------------------------------------------
    def setObjStateModifiedSilent(self, REQUEST):
      setObjState(self,'STATE_MODIFIED_SILENT',REQUEST['lang'])
      self.resetWorkflowObjStates(REQUEST)

    # ---------------------------------------------------------------------------------------------
    #  VersionManager.setObjStateModified
    # ---------------------------------------------------------------------------------------------
    def setObjStateModified(self, REQUEST):
      setChangedBy(self,REQUEST)
      setObjState(self,'STATE_MODIFIED',REQUEST['lang'])
      self.resetWorkflowObjStates(REQUEST)

    # ---------------------------------------------------------------------------------------------
    #  VersionManager.setObjStateDeleted
    # ---------------------------------------------------------------------------------------------
    def setObjStateDeleted(self, REQUEST):
      setChangedBy(self,REQUEST)
      setObjState(self,'STATE_DELETED',REQUEST['lang'])
      self.resetWorkflowObjStates(REQUEST)

    # ---------------------------------------------------------------------------------------------
    #  VersionManager.setObjStateNew
    # ---------------------------------------------------------------------------------------------
    def setObjStateNew(self, REQUEST, reset=1):
      if reset: self.resetObjVersion()
      setChangedBy(self,REQUEST,reset)
      setObjState(self,'STATE_NEW',REQUEST['lang'])
      self.resetWorkflowObjStates(REQUEST)


    """
    ###############################################################################################
    #
    #   C O M M I T
    #
    ###############################################################################################
    """

    # ---------------------------------------------------------------------------------------------
    #  VersionManager.onChangeObj:
    # ---------------------------------------------------------------------------------------------
    def onChangeObj(self, REQUEST, forced=0):
      if self.getAutocommit() or forced:
        parent = self.getParentNode()
        delete = self.commitObjChanges(parent,REQUEST,forced)
        if delete: 
          parent.normalizeSortIdsAfterDelete()


    # ---------------------------------------------------------------------------------------------
    #  VersionManager.onSynchronizeObj:
    # ---------------------------------------------------------------------------------------------
    def onSynchronizeObj(self, REQUEST):
      #-- Catalog
      self.synchronizeSearch(REQUEST)
      #-- Cache
      self.synchronizeCachePage(REQUEST)
      #-- Access
      self.synchronizePublicAccess()
      #-- References
      self.synchronizeRefToObjs()
      self.synchronizeRefByObjs()


    # ---------------------------------------------------------------------------------------------
    #  VersionManager.commitObjChanges
    # ---------------------------------------------------------------------------------------------
    def commitObjChanges(self, parent, REQUEST, forced=0):
      if _globals.debug: print '[%s.commitObjChanges]: %s'%(self.meta_type,self.id)
      delete = 0
      
      ##### Trigger custom onChangeObj-Event (if there is one) ####
      try: 
        if self.meta_type == 'ZMSCustom':
          metaObjAttr = self.getMetaobjAttr(self.meta_id,'onChangeObjEvt')
          if metaObjAttr is not None:
            try: REQUEST.set('preview','preview')
            except: REQUEST['preview'] = 'preview'
            v = self.getObjProperty('onChangeObjEvt',REQUEST)
      except:
        _globals.writeException(self,"[commitObjChanges]: getObjProperty('onChangeObjEvt',REQUEST)")
      try: 
        if hasattr(self,'onChangeObjEvt'):
          try: REQUEST.set('preview','preview')
          except: REQUEST['preview'] = 'preview'
          v = getattr(self,'onChangeObjEvt')(context=self,REQUEST=REQUEST)
      except:
        _globals.writeException(self,"[commitObjChanges]: onChangeObjEvt(self,REQUEST)")
      
      ##### Commit delete object. ####
      if self.inObjStates(['STATE_DELETED'],REQUEST):
        if self.id in parent.objectIds():
          parent.moveObjToTrashcan(self,REQUEST)
          self.resetObjStates(REQUEST)
          delete = 1
          
      ##### Commit modifications. ####
      elif self.getObjStateNames(REQUEST) or forced:
        self.cloneObjAttrs(getattr(self,self.version_work_id),getattr(self,self.version_live_id),REQUEST)
        # Special Objects / Link-Containers.
        children = []
        if self.meta_type == 'ZMSCustom' and not self.isPage():
          children = self.getObjChildren('',REQUEST)
        elif self.meta_type == 'ZMSLinkContainer':
          children = self.getChildNodes(REQUEST,['ZMSLinkElement'])
        for child in children:
          child.commitObjChanges(self,REQUEST)
        # Reset object-state.
        self.resetObjStates(REQUEST)

      ##### Synchronize listeners. ####
      if not delete:
        self.onSynchronizeObj(REQUEST)
        
      # Return flag for deleted objects.
      return delete


    """
    ###############################################################################################
    #
    #   R O L L B A C K
    #
    ###############################################################################################
    """

    # ---------------------------------------------------------------------------------------------
    #  VersionManager.rollbackObjChanges
    # ---------------------------------------------------------------------------------------------
    def rollbackObjChanges(self, parent, REQUEST):
      delete = 0
      auth_user = REQUEST['AUTHENTICATED_USER']
      if str(auth_user) == self.getObjProperty('change_uid',REQUEST) or \
         len(self.intersection_list(self.getUserRoles(auth_user),['Manager','ZMSAdministrator','ZMSEditor'])) > 0:
        ##### Rollback insert object. ####
        if self.inObjStates(['STATE_NEW'],REQUEST):
          if self.id in parent.objectIds():
            parent.moveObjToTrashcan(self,REQUEST)
            self.resetObjStates(REQUEST)
            delete = 1
        ##### Rollback modifications. ####
        elif self.getObjStateNames(REQUEST):
          setChangedBy(self,REQUEST)
          self.cloneObjAttrs(getattr(self,self.version_live_id),getattr(self,self.version_work_id),REQUEST)
          # Special Objects / Link-Containers.
          children = []
          if self.meta_type == 'ZMSCustom' and not self.isPage():
            children = self.getObjChildren('',REQUEST)
          elif self.meta_type == 'ZMSLinkContainer':
            children = self.getChildNodes(REQUEST,['ZMSLinkElement'])
          for child in children:
            child.rollbackObjChanges(self,REQUEST)
          # Reset object-state.
          self.resetObjStates(REQUEST)
      # Return flag for deleted objects.
      return delete


    """
    ###############################################################################################
    #
    #   R E Q U E S T
    #
    ###############################################################################################
    """

    # ---------------------------------------------------------------------------------------------
    #  VersionManager._requestVersionChanges
    # ---------------------------------------------------------------------------------------------
    def _requestVersionChanges(self, parent, lang, manage_lang, REQUEST):
      s = ''
      if len(self.getObjStateNames(REQUEST)) > 0:
        s += '\n* %s: '%self.display_type(REQUEST) 
        for state in ['STATE_NEW','STATE_MODIFIED','STATE_DELETED']:
          if self.inObjStates([state],REQUEST):
            s += '[%s] '%self.getLangStr(state,manage_lang)
        s += '%s %s '%(self.getLangStr('ON',manage_lang),self.getLangFmtDate(self.getObjProperty('change_dt',REQUEST),manage_lang,'DATE_FMT'))
        s += '%s %s '%(self.getLangStr('AT',manage_lang),self.getLangFmtDate(self.getObjProperty('change_dt',REQUEST),manage_lang,'TIME_FMT'))
        s += '%s %s '%(self.getLangStr('BY',manage_lang),self.getObjProperty('change_uid',REQUEST))
      return s


    """
    ###############################################################################################
    #
    #   V E R S I O N
    #
    ###############################################################################################
    """

    ###############################################################################################
    #  VersionManager.manage_UndoVersion:
    #
    #  Undo version changes.
    ###############################################################################################
    manage_UndoVersionForm = HTMLFile('dtml/versionmanager/manage_undoversionform', globals())
    def manage_UndoVersion(self, lang, manage_lang, REQUEST):
      """ VersionManager.manage_UndoVersion """
      message = ''
      
      id = REQUEST.get('id','')
      attrCntnr = getattr(self,id,None)
      if attrCntnr is not None:
        # Restore attributes.
        self.setObjStateModified(REQUEST)
        self.cloneObjAttrs(attrCntnr,getattr(self,self.version_work_id),REQUEST)
        self.onChangeObj(REQUEST)
        # Restore sort-order.
        parent = self.getParentNode()
        if parent is not None: parent.normalizeSortIds(_globals.id_prefix(self.id))
        message = self.getLangStr('MSG_CHANGED',manage_lang)
      
      # Return with message.
      message = urllib.quote(message)
      return REQUEST.RESPONSE.redirect('manage_UndoVersionForm?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(lang,manage_lang,message))



###################################################################################################
###################################################################################################
###
###   class VersionManagerContainer
###
###################################################################################################
###################################################################################################
class VersionManagerContainer: 

    # ---------------------------------------------------------------------------------------------
    #  VersionManagerContainer.resetWorkflowObjStates:
    #
    #  Resets information about workflow-events.
    # ---------------------------------------------------------------------------------------------
    def resetWorkflowObjStates(self, REQUEST):
      lang = REQUEST['lang']
      self.setObjProperty('work_uid',None,lang)
      self.setObjProperty('work_dt',None,lang)
      # Resets information about workflow-events.
      delObjStates(self,['STATE_COMMITTED','STATE_REQUESTED','STATE_ROLLEDBACK','STATE_REJECTED'],REQUEST)

      
    # ---------------------------------------------------------------------------------------------
    #  VersionManagerContainer.normalizeSortIdsAfterDelete:
    #
    #  Resets child-objects after delete..
    # ---------------------------------------------------------------------------------------------
    def normalizeSortIdsAfterDelete(self):
      self.normalizeSortIds()
      # Resets information about silent-modifications.
      for child in self.getChildNodes():
        delObjStates(child,['STATE_MODIFIED_SILENT'])


    """
    ###############################################################################################    
    #
    #   T A S K S 
    #
    ###############################################################################################    
    """

    # Management Interface.
    manage_tasks = HTMLFile('dtml/versionmanager/manage_tasks', globals()) 
    tasklist0 = HTMLFile('dtml/versionmanager/tasklist0', globals()) 
    tasklist1 = HTMLFile('dtml/versionmanager/tasklist1', globals()) 
    tasklist2 = HTMLFile('dtml/versionmanager/tasklist2', globals()) 
    tasklist3 = HTMLFile('dtml/versionmanager/tasklist3', globals()) 
    tasklist4 = HTMLFile('dtml/versionmanager/tasklist4', globals()) 
    tasklist5 = HTMLFile('dtml/versionmanager/tasklist5', globals()) 

    # ---------------------------------------------------------------------------------------------
    #  VersionManagerContainer.getTasks:
    # ---------------------------------------------------------------------------------------------
    def getTasks(self, task_type, REQUEST):
      rtn = []
      append = 1
      if task_type == 0:
        append = append and (self.isObjModified(REQUEST) or self.hasObjModifiedChildren(REQUEST))
      elif task_type == 1:
        append = append and self.inObjStates(['STATE_REQUESTED'],REQUEST)
      elif task_type == 2:
        append = append and self.inObjStates(['STATE_REJECTED'],REQUEST)
      elif task_type == 4:
        translate = len(self.getObjProperty('change_uid',REQUEST)) == 0
        for child in self.getChildNodes(REQUEST):
          translate = translate or len(child.getObjProperty('change_uid',REQUEST)) == 0
        append = append and self.getDCCoverage(REQUEST) not in self.getLangIds() and translate
      if append: rtn.append(self)
      for child in self.getChildNodes(REQUEST,self.PAGES):
        rtn.extend(child.getTasks(task_type,REQUEST))
      return rtn


    """
    ###############################################################################################
    #
    #   C O M M I T
    #
    ###############################################################################################
    """

    # ---------------------------------------------------------------------------------------------
    #  VersionManagerContainer.commitObjSilent:
    #
    #  Auto commit container if it is not modified and nothing but silently
    #  modified objects (sort_id).
    # ---------------------------------------------------------------------------------------------
    def commitObjSilent(self, REQUEST={}):
      #++ if _globals.debug: print '[%s.commitObjSilent]: %s'%(self.meta_type,self.absolute_url())
      req = {'lang':self.getPrimaryLanguage(),'preview':'preview'}
      flag = self.isOnlyModifiedSilent()
      for child in self.getChildNodes(req):
        flag = flag and child.isOnlyModifiedSilent()
      flag = flag or self.getAutocommit()
      if flag:
        self.commitObj()


    # ---------------------------------------------------------------------------------------------
    #  VersionManagerContainer.commitObj
    # ---------------------------------------------------------------------------------------------
    def commitObj(self, REQUEST={}):
      #++ if _globals.debug: print "[%s.commitObj]: id=%s"%(self.meta_type,self.id)
      ##### Children ####
      delete = 0
      for child in self.getChildNodes():
        delete = delete + child.commitObjChanges(self,REQUEST)
      if delete: self.normalizeSortIdsAfterDelete()
      ##### Self ####
      if REQUEST.has_key('lang'): self.resetWorkflowObjStates(REQUEST)
      parent = self.getParentNode()        
      delete = self.commitObjChanges(parent,REQUEST)
      url = self.absolute_url()
      if delete: 
        parent.normalizeSortIdsAfterDelete()
        url = parent.absolute_url()
      # Return new URL.
      return url


    ###############################################################################################
    #  VersionManagerContainer.manage_commitVersionChanges:	
    #
    #  Commit version changes.
    ###############################################################################################
    def manage_commitVersionChanges(self, lang, manage_lang, REQUEST):
      """ VersionManagerContainer.manage_commitVersionChanges """
      
      ##### Notification ####
      
      #-- Recipient
      name = self.getObjProperty('work_uid',REQUEST)
      mto = self.getUserAttr(name, 'email', '')
      #-- Subject
      msubject \
        = self.getLangStr('ACTION_MANAGE_COMMITVERSIONCHANGES',manage_lang) \
        + ' %s "%s"'%(self.display_type(REQUEST),self.getTitle(REQUEST)) \
        + ' %s %s '%(self.getLangStr('BY',manage_lang),str(REQUEST['AUTHENTICATED_USER']))
      #-- Body
      mbody \
        = self.getLangStr('ACTION_MANAGE_COMMITVERSIONCHANGES',manage_lang) \
        + ' %s "%s"'%(self.display_type(REQUEST),self.getTitle(REQUEST)) \
        + ' %s %s '%(self.getLangStr('BY',manage_lang),str(REQUEST['AUTHENTICATED_USER'])) \
        + '\n%s/manage'%(self.absolute_url()) \
        + '\n\n%s\n%s'%(self.getLangStr('WITH_BEST_REGARDS',manage_lang),str(REQUEST['AUTHENTICATED_USER']))
      
      # Send notification via MailHost.
      self.sendMail(mto,msubject,mbody,REQUEST)
      
      ##### Commit ####
      url = self.commitObj(REQUEST)
      
      ##### Set object state ####
      self.resetWorkflowObjStates(REQUEST)
      setObjState(self,'STATE_COMMITTED',lang)
      setWorkflowEventBy(self,REQUEST)
      
      # Return with message.
      message = urllib.quote(self.getLangStr('ACTION_MANAGE_COMMITVERSIONCHANGES',manage_lang))
      return REQUEST.RESPONSE.redirect('%s/manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(url,lang,manage_lang,message))


    """
    ###############################################################################################
    #
    #   R O L L B A C K
    #
    ###############################################################################################
    """

    # ---------------------------------------------------------------------------------------------
    #  VersionManagerContainer.rollbackObj
    # ---------------------------------------------------------------------------------------------
    def rollbackObj(self, REQUEST):
      #++ if _globals.debug: print "[%s.rollbackObj]: id=%s"%(self.meta_type,self.id)
      ##### Children ####
      delete = 0
      for child in self.getChildNodes():
        delete = delete + child.rollbackObjChanges(self,REQUEST)
      if delete: self.normalizeSortIdsAfterDelete()
      ##### Self ####
      if REQUEST.has_key('lang'): self.resetWorkflowObjStates(REQUEST)
      parent = self.getParentNode()
      delete = self.rollbackObjChanges(parent,REQUEST)
      url = self.absolute_url()
      if delete: 
        parent.normalizeSortIdsAfterDelete()
        url = parent.absolute_url()
      # Return new URL.
      return url


    ###############################################################################################
    #  VersionManagerContainer.manage_rejectVersionChanges:
    #
    #  Reject version changes.
    ###############################################################################################
    def manage_rejectVersionChanges(self, lang, manage_lang, REQUEST):
      """ VersionManagerContainer.manage_rejectVersionChanges """
      
      ##### Notification ####
      
      #-- Recipient
      name = self.getObjProperty('work_uid',REQUEST)
      mto = self.getUserAttr(name, 'email', '')
      #-- Subject
      msubject \
        = self.getLangStr('ACTION_MANAGE_REJECTVERSIONCHANGES',manage_lang) \
        + ' %s "%s"'%(self.display_type(REQUEST),self.getTitle(REQUEST)) \
        + ' %s %s '%(self.getLangStr('BY',manage_lang),str(REQUEST['AUTHENTICATED_USER']))
      #-- Body
      mbody \
        = self.getLangStr('ACTION_MANAGE_REJECTVERSIONCHANGES',manage_lang) \
        + ' %s "%s"'%(self.display_type(REQUEST),self.getTitle(REQUEST)) \
        + ' %s %s '%(self.getLangStr('BY',manage_lang),str(REQUEST['AUTHENTICATED_USER'])) \
        + '\n%s/manage'%(self.absolute_url()) \
        + '\n\n%s\n%s'%(self.getLangStr('WITH_BEST_REGARDS',manage_lang),str(REQUEST['AUTHENTICATED_USER']))
      
      # Send notification via MailHost.
      self.sendMail(mto,msubject,mbody,REQUEST)
      
      ##### Set object state ####
      self.resetWorkflowObjStates(REQUEST)
      setObjState(self,'STATE_REJECTED',lang)
      setWorkflowEventBy(self,REQUEST)
      
      # Return with message.
      message = urllib.quote(self.getLangStr('ACTION_MANAGE_REJECTVERSIONCHANGES',manage_lang))
      return REQUEST.RESPONSE.redirect('manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(lang,manage_lang,message))


    ###############################################################################################
    #  VersionManagerContainer.manage_rollbackVersionChanges:	
    #
    #  Rollback version changes.
    ###############################################################################################
    def manage_rollbackVersionChanges(self, lang, manage_lang, REQUEST):
      """ VersionManagerContainer.manage_rollbackVersionChanges """
      
      ##### Notification ####
      
      #-- Recipient
      name = self.getObjProperty('work_uid',REQUEST)
      mto = self.getUserAttr(name,'email','')
      #-- Subject
      msubject \
        = self.getLangStr('ACTION_MANAGE_ROLLBACKVERSIONCHANGES',manage_lang) \
        + ' %s "%s"'%(self.display_type(REQUEST),self.getTitle(REQUEST)) \
        + ' %s %s '%(self.getLangStr('BY',manage_lang),str(REQUEST['AUTHENTICATED_USER']))
      #-- Body
      mbody \
        = self.getLangStr('ACTION_MANAGE_ROLLBACKVERSIONCHANGES',manage_lang) \
        + ' %s "%s"'%(self.display_type(REQUEST),self.getTitle(REQUEST)) \
        + ' %s %s '%(self.getLangStr('BY',manage_lang),str(REQUEST['AUTHENTICATED_USER'])) \
        + '\n%s/manage'%(self.absolute_url()) \
        + '\n\n%s\n%s'%(self.getLangStr('WITH_BEST_REGARDS',manage_lang),str(REQUEST['AUTHENTICATED_USER']))
      
      # Send notification via MailHost.
      self.sendMail(mto,msubject,mbody,REQUEST)
      
      ##### Rollback ####
      url = self.rollbackObj(REQUEST)
      
      ##### Set object state ####
      self.resetWorkflowObjStates(REQUEST)
      setObjState(self,'STATE_ROLLEDBACK',lang)
      setWorkflowEventBy(self,REQUEST)
      self.commitObjSilent(REQUEST)
      
      # Return with message.
      message = urllib.quote(self.getLangStr('ACTION_MANAGE_ROLLBACKVERSIONCHANGES',manage_lang))
      return REQUEST.RESPONSE.redirect('%s/manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(url,lang,manage_lang,message))


    """
    ###############################################################################################
    #
    #   R E Q U E S T
    #
    ###############################################################################################
    """

    ###############################################################################################
    #  VersionManagerContainer.manage_requestVersionChanges:
    #
    #  Request commit of version changes.
    ###############################################################################################
    def manage_requestVersionChanges(self, lang, manage_lang, REQUEST):
      """ VersionManagerContainer.manage_requestVersionChanges """
      
      ##### Set object state ####
      self.resetWorkflowObjStates(REQUEST)
      setObjState(self,'STATE_REQUESTED',lang)
      setWorkflowEventBy(self,REQUEST)
      
      ##### Notifications ####
      
      #-- Recipients
      recipients = []
      role = 'ZMSEditor'
      if role in self.getUserRoles(REQUEST['AUTHENTICATED_USER']):
        role = 'ZMSAdministrator'
      ob = self
      while ob is not None and len(recipients) == 0:
        for local_role in ob.get_local_roles():
          name = local_role[0]
          userObj = _accessmanager.findUser(self, name)
          mto = self.getUserAttr(name,'email','')
          if len(mto) > 0 and \
             role in ob.getUserRoles(userObj, aq_parent=0) and \
             lang in ob.getUserLangs(userObj, aq_parent=0):
            recipients.append({'userid':name,'email':mto})
        ob = ob.getParentNode()
      
      #-- Body
      
      # Init message body.
      mbody \
        = self.getLangStr('ACTION_MANAGE_REQUESTVERSIONCHANGES',manage_lang) \
        + ' %s "%s"'%(self.display_type(REQUEST),self.getTitle(REQUEST)) \
        + ' %s %s '%(self.getLangStr('BY',manage_lang),str(REQUEST['AUTHENTICATED_USER'])) \
        + '\n%s/manage'%(self.absolute_url())
      # Build message body.
      mbody += self._requestVersionChanges(self,lang,manage_lang,REQUEST)
      mbody += '\n'
      for child in self.getChildNodes():
        mbody += child._requestVersionChanges(self,lang,manage_lang,REQUEST)
      # With best regards.                
      mbody += '\n\n%s\n%s'%(self.getLangStr('WITH_BEST_REGARDS',manage_lang),str(REQUEST['AUTHENTICATED_USER']))
      
      #-- Subject
      msubject \
        = self.getLangStr('ACTION_MANAGE_REQUESTVERSIONCHANGES',manage_lang) \
        + ' %s "%s"'%(self.display_type(REQUEST),self.getTitle(REQUEST)) \
        + ' %s %s '%(self.getLangStr('BY',manage_lang),REQUEST['AUTHENTICATED_USER'])
      
      # Send notifications via MailHost.
      for recipient in recipients:
        mto = recipient['email']
        self.sendMail(mto,msubject,mbody,REQUEST)
      
      # Return with message.
      message = urllib.quote(self.getLangStr('ACTION_MANAGE_REQUESTVERSIONCHANGES',manage_lang))
      return REQUEST.RESPONSE.redirect('manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(lang,manage_lang,message))

###################################################################################################
