###################################################################################################
# _workflowmanager.py
#
# $Id: _workflowmanager.py,v 1.2 2003/10/05 13:00:11 dnordmann Exp $
# $Name:  $
# $Author: dnordmann $
# $Revision: 1.2 $
#
# Implementation of class WorkflowManager (see below).
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from __future__ import nested_scopes
from Globals import HTMLFile
import ZPublisher.HTTPRequest
import copy
import urllib
# Product Imports.
import _blobfields
import _globals
import _versionmanager


# -------------------------------------------------------------------------------------------------
#  _workflowmanager.importXml
# -------------------------------------------------------------------------------------------------
def importXml(self, xml, REQUEST=None):
  v = self.parseXmlString(xml)
  self.setConfProperty('ZMS.workflow.items',copy.copy(v))

# ---------------------------------------------------------------------------------------------
#  _workflowmanager.setWorkflowItem
# ---------------------------------------------------------------------------------------------
def setWorkflowItem(self, id, newId, newName, newIcon=None):
  obs = self.getConfProperty('ZMS.workflow.items',[])
  # Remove exisiting entry.
  if id in obs:
    i = obs.index(id)
    del obs[i] 
    del obs[i] 
  else: 
    i = len(obs)
  # Values.
  newValues = {}
  newValues['name'] = newName
  newValues['icon'] = newIcon
  # Update attribute.
  obs.insert(i,newValues)
  obs.insert(i,newId)
  self.setConfProperty('ZMS.workflow.items',copy.copy(obs))

# ---------------------------------------------------------------------------------------------
#  _workflowmanager.delWorkflowItem
# ---------------------------------------------------------------------------------------------
def delWorkflowItem(self, id):
  obs = self.getConfProperty('ZMS.workflow.items',[])
  # Update attribute.
  if id in obs:
    i = obs.index(id)
    del obs[i] 
    del obs[i] 
  self.setConfProperty('ZMS.workflow.items',copy.copy(obs))

# ---------------------------------------------------------------------------------------------
#  _workflowmanager.moveWorkflowItem
# ---------------------------------------------------------------------------------------------
def moveWorkflowItem(self, id, dir):
  obs = self.getConfProperty('ZMS.workflow.items',[])
  # Move.
  i = obs.index(id)
  id = obs[i] 
  values = obs[i+1]
  del obs[i] 
  del obs[i] 
  obs.insert(i+dir*2,values)
  obs.insert(i+dir*2,id)
  # Update attribute.
  self.setConfProperty('ZMS.workflow.items',copy.copy(obs))

# ---------------------------------------------------------------------------------------------
#  _workflowmanager.setWorkflowItemAttr
# ---------------------------------------------------------------------------------------------
def setWorkflowItemAttr(self, id, newNextState, newNextRoles, newNextMethod):
  obs = self.getConfProperty('ZMS.workflow.items',[])
  i = obs.index(id)
  ob = obs[i+1]
  attrs = copy.copy(ob.get('attrs',[]))
  # Add.
  attr = {}
  attr['next_state'] = newNextState
  attr['next_roles'] = newNextRoles
  attr['next_method'] = newNextMethod
  attrs.append(attr)
  # Update attribute.
  ob['attrs'] = attrs
  obs[i+1] = ob
  self.setConfProperty('ZMS.workflow.items',copy.copy(obs))

# ---------------------------------------------------------------------------------------------
#  _workflowmanager.delWorkflowItemAttr
# ---------------------------------------------------------------------------------------------
def delWorkflowItemAttr(self, id, attr):
  obs = self.getConfProperty('ZMS.workflow.items',[])
  i = obs.index(id)
  ob = obs[i+1]
  attrs = copy.copy(ob.get('attrs',[]))
  # Delete.
  del attrs[attr]
  # Update attribute.
  ob['attrs'] = attrs
  obs[i+1] = ob
  self.setConfProperty('ZMS.workflow.items',copy.copy(obs))

# ---------------------------------------------------------------------------------------------
#  _workflowmanager.moveWorkflowItemAttr
# ---------------------------------------------------------------------------------------------
def moveWorkflowItemAttr(self, id, attr, dir):
  obs = self.getConfProperty('ZMS.workflow.items',[])
  i = obs.index(id)
  ob = obs[i+1]
  attrs = copy.copy(ob.get('attrs',[]))
  # Move.
  v = attrs[attr] 
  del attrs[attr] 
  attrs.insert(attr+dir,v)
  # Update attribute.
  ob['attrs'] = attrs
  obs[i+1] = ob
  self.setConfProperty('ZMS.workflow.items',copy.copy(obs))


###################################################################################################
###################################################################################################
###
###   class WorkflowManager
###
###################################################################################################
###################################################################################################
class WorkflowManager: 

    # ---------------------------------------------------------------------------------------------
    #  WorkflowManager.getAutocommit
    # ---------------------------------------------------------------------------------------------
    def getAutocommit(self): 
      return self.getConfProperty('ZMS.autocommit',1)

    # ---------------------------------------------------------------------------------------------
    #  WorkflowManager.doAutocommit:
    #
    #  Auto-Commit ZMS-tree.
    # ---------------------------------------------------------------------------------------------
    def doAutocommit(self, lang, manage_lang, REQUEST):
      _versionmanager.doAutocommit(self,REQUEST)

    # ---------------------------------------------------------------------------------------------
    #  WorkflowManager.getWorkflowItems
    # ---------------------------------------------------------------------------------------------
    def getWorkflowItems(self): 
      obs = self.getConfProperty('ZMS.workflow.items',[])
      workflowItms = []
      for i in range(len(obs)/2):
        id = obs[i*2]
        workflowItm = obs[i*2+1].copy()
        workflowItm['id'] = id
        workflowItms.append(workflowItm)
      return workflowItms

    # ---------------------------------------------------------------------------------------------
    #  WorkflowManager.getWorkflowItem
    # ---------------------------------------------------------------------------------------------
    def getWorkflowItem(self, id):
      obs = self.getConfProperty('ZMS.workflow.items',[])
      i = obs.index(id)
      workflowItm = obs[i+1].copy()
      workflowItm['id'] = id
      return workflowItm
    

    ###############################################################################################
    #  WorkflowManager.manage_customizeWorkflow:
    #
    #  Customize workflow.
    ###############################################################################################
    manage_customizeWorkflowForm = HTMLFile('dtml/zms/manage_customizeworkflowform', globals())
    def manage_changeWorkflow(self, lang, manage_lang, btn='', key='', REQUEST=None, RESPONSE=None):
      """ WorkflowManager.manage_changeWorkflow """
      message = ''
      id = REQUEST.get('id','')
      
      # Change.
      # -------
      if btn == self.getLangStr('BTN_CHANGE',manage_lang):
      
        # Autocommit.
        if key == 'autocommit':
          old_autocommit = self.getAutocommit()
          new_autocommit = REQUEST.get('workflow',0)==0
          self.setConfProperty('ZMS.autocommit',new_autocommit)
          if old_autocommit == 0 and new_autocommit == 1:
            self.doAutocommit(lang,manage_lang,REQUEST)
          message = self.getLangStr('MSG_CHANGED',manage_lang)

        # Obj.
        else:
          workflowItm = self.getWorkflowItem(id)
          id = REQUEST.get('inpId').strip()
          name = REQUEST.get('inpName').strip()
          icon = REQUEST.get('inpIcon','')
          if hasattr(icon,'__class__') and getattr(icon,'__class__') is ZPublisher.HTTPRequest.FileUpload:
            if len(getattr(icon,'filename',''))==0:
              icon = workflowItm.get('icon',None)
            else:
              icon = _blobfields.createBlobField(self,_globals.DT_IMAGE,icon)
          setWorkflowItem(self,workflowItm['id'],id,name,icon)
          message = self.getLangStr('MSG_CHANGED',manage_lang)

      # Delete.
      # -------
      elif btn in ['delete',self.getLangStr('BTN_DELETE',manage_lang)]:

        # Obj.
        if key == 'obj':
          delWorkflowItem(self,id)
          id = ''
          message = self.getLangStr('MSG_CHANGED',manage_lang)
      
        # Attr.
        elif key == 'attr':
          attr = int(REQUEST['attr_id'])
          delWorkflowItemAttr(self,id,attr)
          message = self.getLangStr('MSG_CHANGED',manage_lang)

      # Insert.
      # -------
      elif btn == self.getLangStr('BTN_INSERT',manage_lang):
        
        # Obj.
        if key == 'obj':
          id = REQUEST.get('newId').strip()
          name = REQUEST.get('newName').strip()
          setWorkflowItem(self,None,id,name)
          message = self.getLangStr('MSG_INSERTED',manage_lang)%id

        # Attr.
        elif key == 'attr':
          newNextState = REQUEST.get('newNextState').strip()
          newNextRoles = REQUEST.get('newNextRoles',[])
          newNextMethod = REQUEST.get('newNextMethod').strip()
          setWorkflowItemAttr(self,id,newNextState,newNextRoles,newNextMethod)
          message = self.getLangStr('MSG_INSERTED',manage_lang)%newNextState

      # Export.
      # -------
      elif btn == self.getLangStr('BTN_EXPORT',manage_lang):
        value = self.getConfProperty('ZMS.workflow.items',[])
        content_type = 'text/xml'
        filename = 'workFlow.xml'
        export = '<?xml version="1.0" encoding="ISO-8859-1"?>'
        export = export + self.toXmlString(value,1)
        RESPONSE.setHeader('Content-Type',content_type)
        RESPONSE.setHeader('Content-Disposition','inline;filename=%s'%filename)
        return export

      # Import.
      # -------
      elif btn == self.getLangStr('BTN_IMPORT',manage_lang):
        f = REQUEST['file']
        importXml(self,xml=f)
        message = self.getLangStr('MSG_IMPORTED',manage_lang)%('<i>%s</i>'%f.filename)
      
      # Move up.
      # --------
      elif btn == 'move_up':

        # Obj.
        if key == 'obj':
          moveWorkflowItem(self,id,-1)
          message = self.getLangStr('MSG_MOVEDOBJUP',manage_lang)%("<i>%s</i>"%id)
          id = ''
        
        # Attr.
        elif key == 'attr':
          attr = int(REQUEST['attr_id'])
          moveWorkflowItemAttr(self,id,attr,-1)
          message = self.getLangStr('MSG_MOVEDOBJDOWN',manage_lang)%("<i>%s</i>"%attr)
        
      # Move down.
      # ----------
      elif btn == 'move_down':

        # Obj.
        if key == 'obj':
          moveWorkflowItem(self,id,+1)
          message = self.getLangStr('MSG_MOVEDOBJDOWN',manage_lang)%("<i>%s</i>"%id)
          id= ''

        # Attr.
        elif key == 'attr':
          attr = int(REQUEST['attr_id'])
          moveWorkflowItemAttr(self,id,attr,+1)
          message = self.getLangStr('MSG_MOVEDOBJDOWN',manage_lang)%("<i>%s</i>"%attr)
        
      # Return with message.
      message = urllib.quote(message)
      return RESPONSE.redirect('manage_customizeWorkflowForm?id=%s&lang=%s&manage_lang=%s&manage_tabs_message=%s'%(id,lang,manage_lang,message))

###################################################################################################
